C$Procedure      RDMISN ( Read the mission template file. )
 
      SUBROUTINE RDMISN ( MTEMPL, KWTBLE, KWPTR, KWVALS )
 
C$ Abstract
C
C     Read a mission template file for generating a set of labels.
C     This subroutine is for use ONLY with the maklabel program.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     None.
C
C$ Declarations
 
      IMPLICIT NONE
      INTEGER               LBCELL
      PARAMETER           ( LBCELL = -5 )
      CHARACTER*(*)         MTEMPL
      CHARACTER*(*)         KWTBLE ( LBCELL: * )
      INTEGER               KWPTR  ( LBCELL: * )
      CHARACTER*(*)         KWVALS ( LBCELL: * )
 
      INTEGER               KWDLEN
      PARAMETER           ( KWDLEN = 80 )
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     MTEMPL     I   The name of the mission template file.
C     NMISKW     I   The number of mission specific labels in the file.
C     MISKWD     I   The keywords we expect to find in the file.
C     VALUES     O   The values associated with each keyword.
C     INPLEN     P   The length of an input line.
C     KWDLEN     P   The length of a keyword.
C
C$ Detailed_Input
C
C     MTEMPL   The name of the mission template file. This must be a
C              full path name. This file contains the mission specific
C              keyword-value pairs in the following format:
C
C                 <keyword 1> = <value 1>
C                 <keyword 2> = <value 2>
C                           .
C                           .
C                           .
C                 <keyword n> = <value n>
C
C              with an optional "end" line containing only the word
C              'END'.
C
C     NMISKW   The number of mission specific labels in the file.
C
C     MISKWD   The keywords we expect to find in the file. Only these
C              keywords may appear in the file. If any other keywords
C              are encountered an error will be signalled.
C
C$ Detailed_Output
C
C     VALUES   The values associated with each keyword. Upon successful
C              completion, this array will contain the values for the
C              mission specific keywords in corresponding locations,
C              i.e., the value for MISKWD(I) is in the location
C              VALUES(I).
C
C              It is an error if a value read from the mission template
C              file is too long to fit into an element of the VALUES
C              array.
C
C$ Parameters
C
C     INPLEN   The length of a line read from the mission template file.
C
C     KWDLEN   The length of a keyword.
C
C$ Exceptions
C
C
C     1) If the mission template file does not exist, the error
C        MAKLABEL(FILENOTFOUND) will be signalled.
C
C     2) If a mission specific keyword that is not contained in the
C        array MISKWD is encountered, the error MAKLABEL(BADKEYWORD)
C        will be signalled.
C
C     3) If a mission specific keyword that is in the array MISKWD is
C        not found in the mission template file the error
C        MAKLABEL(KEYWORDNOTFOUND) will be signalled.
C
C     4) If the value for a keyword is blank, the error
C        MAKLABEL(BLANKVALUE) will be signalled.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     None.
C
C$ Examples
C
C     None.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     K.R. Gehringer     (JPL)
C
C-    Beta Version 1.0.0, 13-NOV-1994 (KRG)
C
C-&
 
C$ Index_Entries
C
C      maklabel read mission template file
C
C-&
C
C     SPICELIB functions
C
      INTEGER               RTRIM
 
      LOGICAL               EXISTS
      LOGICAL               FAILED
      LOGICAL               RETURN
      LOGICAL               ELEMC
 
C
C     Local variables
C
      INTEGER               LNSIZE
      PARAMETER           ( LNSIZE = 132 )
 
      INTEGER               LNGSIZ
      PARAMETER           ( LNGSIZ = 320 )
 
      INTEGER               WDSIZE
      PARAMETER           ( WDSIZE = 32 )
 
      CHARACTER*(LNGSIZ)    DIAGNS
      CHARACTER*(KWDLEN)    KEY
      CHARACTER*(LNSIZE)    LINE
      CHARACTER*(LNSIZE)    ULINE
      CHARACTER*(LNSIZE)    VALUE
 
 
 
      INTEGER               NCTRLS
      PARAMETER           ( NCTRLS = 2 )
 
      CHARACTER*(WDSIZE)    CTRLS ( LBCELL:NCTRLS )
 
 
      CHARACTER*(1)         EQUAL
      CHARACTER*(1)         RDCHAR
      CHARACTER*(1)         LDCHAR
      CHARACTER*(1)         TAB
      CHARACTER*(1)         SPACE
 
      INTEGER               MISLUN
      INTEGER               BVAR
      INTEGER               EVAR
      INTEGER               BVAL
      INTEGER               EVAL
      INTEGER               I
      INTEGER               L
      INTEGER               ASSIGN
      INTEGER               RDELIM
      INTEGER               LDELIM
      INTEGER               BLANK
      INTEGER               LINNUM
 
      LOGICAL               DONE
      LOGICAL               EOF
      LOGICAL               EXAMIN
      LOGICAL               OK
      LOGICAL               CMPLTE
 
      LOGICAL               BEGIN
      LOGICAL               INVAR
      LOGICAL               BLANK1
      LOGICAL               NDDLM1
      LOGICAL               NDCHAR
      LOGICAL               NDRDLM
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      END IF
 
      CALL CHKIN ( 'RDMISN' )
 
C
C     Be sure that the file exists before we try to open it. This also
C     detects a blank filename.
C
      IF ( .NOT. EXISTS(MTEMPL) ) THEN
         CALL SETMSG ( 'The mission template file ''#'' does'
     .   //            ' not exist.'                          )
         CALL SIGERR ( 'MAKLABEL(NOSUCHFILE)'                 )
         CALL CHKOUT ( 'RDMISN'                               )
         RETURN
      END IF
C
C     Ok. Now set up the various items we'll need for the
C     processing of the template file.
C
      CTRLS ( 1 ) = ' '
      CTRLS ( 2 ) = 'END'
 
      CALL VALIDC ( NCTRLS, NCTRLS, CTRLS )
 
      EQUAL  = '='
      RDCHAR = '"'
      LDCHAR = '"'
      SPACE  = ' '
      TAB    = CHAR(9)
 
      ASSIGN = ICHAR( EQUAL  )
      RDELIM = ICHAR( RDCHAR )
      LDELIM = ICHAR( LDCHAR )
      BLANK  = ICHAR( ' '    )
 
C
C     Open the mission template file.
C
      CALL TXTOPR ( MTEMPL, MISLUN )
 
      IF ( FAILED() ) THEN
         CALL CHKOUT ( 'RDMISN' )
         RETURN
      END IF
C
C     While we are not at the end of the file, read in a
C     keyword-value pair, which MUST be on a single line of
C     at most INPLEN characters, and process it, setting a
C     value or signalling an error.
C
 
      CALL READLN ( MISLUN, LINE, EOF )
      LINNUM = 1
 
      IF ( FAILED() ) THEN
         CALL CHKOUT ( 'RDMISN' )
         RETURN
      END IF
 
      DONE   =  EOF
      EXAMIN = .TRUE.
 
      DO WHILE ( .NOT. DONE )
 
C
C        Left justify the line.
C
         CALL REPLCH ( LINE, TAB, SPACE, LINE )
         CALL LJUST  ( LINE, LINE  )
         CALL UCASE  ( LINE, ULINE )
C
C        Certain lines are control lines.  We look for
C        those and act accordingly.
C
         OK     = .TRUE.
 
         IF ( ELEMC(ULINE, CTRLS) ) THEN
 
            IF ( LINE .EQ. 'END' ) THEN
               DONE   = .TRUE.
               EXAMIN = .FALSE.
            END IF
 
         ELSE IF ( EXAMIN ) THEN
C
C           So far we do not have a complete keyword = value
C           specification in this line.
C
            CMPLTE = .FALSE.
 
C
C           All keyword value pairs should look like
C
C           VARIABLE = "VALUE"
C
C           VARIABLE should be a consecutive sequence of non-blank
C           letters and should be composed of the characters other
C           than the delimiter character, and the assignment
C           character.
C
C           Test this line to see that it follows this pattern. We
C           perform this test via a finite state machine.  The
C           state machine examines each character of the line as the
C           state progresses from
C
C               Begin to
C               Reading Keyword to
C               Looking for assignment character to
C               Looking for left delimiter of value to
C               Looking for at first character of value to
C               Looking for a right delimiter of the value.
C
C           At various stages we keep track of the beginning
C           and endings of the keyword name and associated value.
C
            BEGIN  = .TRUE.
            INVAR  = .FALSE.
            BLANK1 = .FALSE.
            NDDLM1 = .FALSE.
            NDCHAR = .FALSE.
            NDRDLM = .FALSE.
 
 
            DO I = 1, RTRIM(LINE)
 
               L = ICHAR( LINE(I:I) )
 
               IF ( BEGIN ) THEN
C
C                 We get here by being at the beginning of the
C                 input line.  We shall not return this way
C                 until the next line.
C
                  BEGIN  = .FALSE.
                  INVAR  = .TRUE.
                  BVAR   =  I
                  EVAR   =  I
                  OK     =       L .NE. ASSIGN
     .                     .AND. L .NE. LDELIM
     .                     .AND. L .NE. RDELIM
 
                  IF ( .NOT. OK ) THEN
 
                     DIAGNS = 'The first character of line # was '
     .               //       '''@''.  This is not a valid '
     .               //       'character for the beginning of a '
     .               //       'keyword. '
                     CALL REPMC ( DIAGNS, '@', LINE(I:I), DIAGNS )
 
                  END IF
 
               ELSE IF ( INVAR ) THEN
C
C                 We get to this point by getting an acceptable
C                 first character and we keep returning to this
C                 block until we hit a blank or the assignment
C                 character.
C
                  OK =       L .NE. LDELIM
     .                 .AND. L .NE. RDELIM
 
                  IF ( OK .AND.  L .NE. BLANK
     .                    .AND.  L .NE. ASSIGN ) THEN
 
                     EVAR = I
 
                  ELSE IF ( L .EQ. BLANK ) THEN
 
                     BLANK1 = .TRUE.
                     INVAR  = .FALSE.
 
                  ELSE IF ( L .EQ. ASSIGN ) THEN
 
                     NDDLM1 = .TRUE.
                     INVAR  = .FALSE.
                     BLANK1 = .FALSE.
 
                  ELSE
 
                     DIAGNS = 'Line #. The delimiter ''@'' must '
     .               //       'follow the assignment character '
     .               //       '''@'' '
                     CALL REPMC ( DIAGNS, '@', LINE(I:I), DIAGNS )
                     CALL REPMC ( DIAGNS, '@', EQUAL,     DIAGNS )
 
                  END IF
 
               ELSE IF ( BLANK1 ) THEN
C
C                 We get here only if we hit a blank while
C                 reading the variable name.  We keep returning
C                 to this point until we hit the assignment
C                 character.
C
                  OK =     L .EQ. BLANK
     .                .OR. L .EQ. ASSIGN
 
                  BLANK1 = L .EQ. BLANK
                  NDDLM1 = L .EQ. ASSIGN
 
                  IF ( .NOT. OK ) THEN
 
                     DIAGNS = 'Line #. The only character that '
     .               //       'may occur between a keyword and '
     .               //       'the assignment character ''@'' is '
     .               //       'the blank character. '
 
                     CALL REPMC ( DIAGNS, '@', EQUAL,     DIAGNS )
                  END IF
 
               ELSE IF ( NDDLM1 ) THEN
C
C                 We can reach this point only if we have encountered
C                 the assignment character.  We will continue
C                 to return to this point until we hit the left
C                 delimiter character.
C
                  OK  =      L .EQ. BLANK
     .                  .OR. L .EQ. LDELIM
 
                  NDDLM1 = L .EQ. BLANK
                  NDCHAR = L .EQ. LDELIM
 
                  IF ( .NOT. OK ) THEN
 
                     DIAGNS = 'Line #. The only character that '
     .               //       'may occur between the assignment '
     .               //       'character ''@'' and left value '
     .               //       'delimiter ''@'' is the blank '
     .               //       'character. '
 
                     CALL REPMC ( DIAGNS, '@', EQUAL,     DIAGNS )
                     CALL REPMC ( DIAGNS, '@', LDCHAR,    DIAGNS )
 
                  END IF
 
               ELSE IF ( NDCHAR ) THEN
C
C                 We reach this block only after hitting the
C                 left delimiter character.  We shall not return
C                 to this block again during the examination of
C                 the current line.
C
                  OK     =  L .NE. RDELIM
                  NDCHAR = .FALSE.
                  NDRDLM = .TRUE.
                  BVAL   =  I
                  EVAL   =  I
 
                  IF ( .NOT. OK ) THEN
                     DIAGNS = 'Line #.  There must be at least '
     .               //       'one character between the left '
     .               //       'value delimiter ''@'' and the '
     .               //       'right value delimiter ''@''. '
                     CALL REPMC ( DIAGNS, '@', LDCHAR,    DIAGNS )
                     CALL REPMC ( DIAGNS, '@', RDCHAR,    DIAGNS )
                  END IF
 
               ELSE IF ( NDRDLM ) THEN
C
C                 We get here by  getting the first character
C                 of a value string.  We continue to return to this
C                 block until we hit the right delimiter character.
C
                  NDRDLM = L .NE. RDELIM
 
                  IF ( NDRDLM ) THEN
                     EVAL = I
                  ELSE
                     OK     =  I .EQ. RTRIM(LINE)
                     CMPLTE = .TRUE.
                  END IF
 
                  IF ( .NOT. OK ) THEN
 
                     DIAGNS = 'Line #.  Extra characters appear '
     .               //       'after the value of the keyword. '
 
                  END IF
 
               END IF
 
               IF ( .NOT. OK ) THEN
                  CALL SETMSG ( DIAGNS                 )
                  CALL ERRINT ( '#', LINNUM            )
                  CALL SIGERR ( 'MAKLABEL(BADKEYWORD)' )
                  CALL CHKOUT ( 'RDMISN' )
                  RETURN
               END IF
 
            END DO
 
C
C           If we didn't complete the keyword assignment we
C           signal an error.
C
            IF ( .NOT. CMPLTE ) THEN
 
               CALL SETMSG ( 'Line # does not match the pattern '
     .         //            '''KEYWORD # #value#'' . ' )
               CALL ERRINT ( '#', LINNUM )
               CALL ERRCH  ( '#', EQUAL  )
               CALL ERRCH  ( '#', LDELIM )
               CALL ERRCH  ( '#', RDELIM )
               CALL SIGERR ( 'MAKLABEL(BADKEYWORD)' )
               CALL CHKOUT ( 'RDMISN' )
               RETURN
 
            END IF
 
C
C           Everything appears to be ok with this assignment.
C           Add it to the keyword table.
C
            KEY   = LINE(BVAR:EVAR)
            VALUE = LINE(BVAL:EVAL)
 
 
            CALL UCASE  ( KEY, KEY )
            CALL SYSETC ( KEY, VALUE, KWTBLE, KWPTR, KWVALS )
 
         END IF
C
C        Fetch the next line
C
 
         CALL READLN ( MISLUN, LINE, EOF )
 
         DONE   = DONE .OR. EOF
         LINNUM = LINNUM + 1
 
      END DO
 
      CLOSE ( MISLUN )
 
      CALL CHKOUT ( 'RDMISN' )
      RETURN
 
      END
